package net.stickycode.coercion.resource;

import static net.stickycode.exception.Preconditions.notNull;

import java.io.InputStream;
import java.net.URI;

import net.stickycode.coercion.CoercionTarget;
import net.stickycode.resource.ResourceLocation;
import net.stickycode.resource.ResourceProtocol;

public class UriResourceLocation
    implements ResourceLocation {

  private ResourceProtocol protocol;

  private URI uri;

  private CoercionTarget type;

  public UriResourceLocation(CoercionTarget type, ResourceProtocol protocol, URI uri) {
    this.type = notNull(type, "A resource location must have a target");
    this.uri = notNull(uri, "A resource location must have a uri");
    this.protocol = notNull(protocol, "A url resource location must have a protocol");
  }

  public ResourceProtocol getProtocol() {
    return protocol;
  }

  public URI getUri() {
    return uri;
  }

  @Override
  public CoercionTarget getResourceTarget() {
    return type;
  }

  @Override
  public InputStream getInputStream() {
    return protocol.getInputStream(this);
  }

  @Override
  public String getPath() {
    if (uri.getPath().length() == 0)
      return uri.getHost();

    return uri.getPath();
  }

}
